"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const recompose_1 = require("recompose");
const withUpdating = ({ component: LoadingComponent, updatingStyle, updatingName = 'isUpdating' }) => WrappedComponent => props => {
    const mergedStyle = Object.assign({}, styles.updating, { updatingStyle });
    return (react_1.default.createElement("div", { style: styles.container },
        react_1.default.createElement(WrappedComponent, Object.assign({}, props)),
        props[updatingName] && react_1.default.createElement(LoadingComponent, { style: mergedStyle })));
};
const styles = {
    container: {
        position: 'relative'
    },
    updating: {
        position: 'fixed',
        top: 0,
        left: 0,
        height: '100%',
        display: 'flex',
        width: '100%',
        background: 'rgba(255,255,255,0.5)'
    }
};
const simulatePending = updates => props => updates.reduce((acc, updatePropName) => {
    acc[updatePropName] = (...args) => __awaiter(this, void 0, void 0, function* () {
        try {
            props.setUpdating(true);
            const response = yield props[updatePropName](...args);
            props.setUpdating(false);
            return response;
        }
        catch (err) {
            props.setUpdating(false);
            throw err;
        }
    });
    return acc;
}, {});
const withManualUpdating = ({ component, updatingStyle }) => recompose_1.compose(recompose_1.withState('isManualUpdating', 'setManualUpdating', false), withUpdating({
    component,
    updatingStyle,
    updatingName: 'isManualUpdating'
}));
exports.withManualUpdating = withManualUpdating;
const withUpdatingCreator = ({ updates, component, updatingStyle }) => recompose_1.compose(recompose_1.withState('isUpdating', 'setUpdating', false), recompose_1.withProps(simulatePending(updates)), withUpdating({
    component,
    updatingStyle
}));
exports.withUpdatingCreator = withUpdatingCreator;
